%*%* This model was published as Hoyle, R.B. & Ezard, T.H.G. (2012). 
%*%* The benefits of maternal effects in novel and in stable environments. 
%*%* J. Roy. Soc. Interface 9, 2403-2413. doi:10.1098/rsif.2012.0183

%*%* The code modifies the reaction norm approach of Lande (2009) to include maternal effects ...
%*%* ... and uses parameters from Lande 2009 and Chevin & Lande 2010 where appropriate.

%*%* SOME CONDITIONS FOR THE MODEL ASSUMPTIONS
%*%* phi=(Gbb*delta^2)/(Gaa + Gbb*delta^2) NEAR 1
%*%* sig_xi^2/delta^2 AND gamma*E(sig_z^2) SMALL
%*%*  ...where sig_z^2 = Gaa + 2*Gab*eps_tT + Gbb*exp_tT^2 + sig_e2

%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
close;
clear;

%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
%*%* parameter definitions
tau=0.75;       	% fraction of a generation between critical period of development and selection 
					% note that in this code any 0<tau<1 has the same effect
rho=0.25;			% environmental auto-correlation
xi_bar=0;			% mean of stationary autocorrelated noise
sig_xi=2;			% standard deviation of stationary autocorrelated noise
omega2=50;			% width of fitness function, 3=> strong stabilizing selection
sigma_e2=.5;		% variance in residual component of phenotypic variation
A=0;				% intercept (elevation) of evolution in reference environment
B=2;				% slope of reaction norm in reference environment
Wmax=1;				% maximal fitness
m=0.5;				% m>0  => maternal effects
noPlas=false;		% holds bbar at 0 for the special case included in fig 2 of Lande 2009
expec=false;		% forces xi the to be 0 for all environments
Gaa=0.5;			% variance of elevation
Gab=0;				% assume zero covariance 
Gba=0; 				% assume zero covariance 
Gbb=0.045;			% variance of plasticity
Ut=10;				% time at which step change of size delta occurs
delta=10;			% size of step change
nrp=1e5;			% length of simulation
maxm=3;
mvals=[0.8 0.45 0];	%different levels of m to explore
 
%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
%*%* define things to receive variables
xi_devel=zeros(nrp,1);
xi_select=zeros(nrp,1);
eabar=zeros(nrp,1);
ebbar=zeros(nrp,1);
ezbar=zeros(nrp,1);
ezsbr=zeros(nrp,1);
efitness=zeros(nrp,1);
epst=zeros(nrp,1);
epstT=zeros(nrp,1);

GG=[Gaa Gab; Gba Gbb];

c1=[0.5 0.5 0.5];
r1=c1(1);
g1=c1(2);
b1=c1(3);
c2=[0 0 0];
r2=c2(1);
g2=c2(2);
b2=c2(3);

colormap=[linspace(r1, r2, maxm)' linspace(g1, g2, maxm)' linspace(b1, b2, maxm)'];

%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
%*%* run simulation by looping to nrp over each of the mvals 

for mm=1:maxm
  m=mvals(mm);

  %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
  %*%* initiate
  eabar(1)=(1-m)*A;
  eabar(2)=(1-m)*A;
  ebbar(1)=rho*B/(1+m^2);
  ebbar(2)=rho*B/(1+m^2);
  ezsbr(1)=A;
  ezbar(1)=A;

  vxi=sig_xi^2;
  esigz2=Gaa*(2+m)/(1-m^2)/(2-m)+(sigma_e2+vxi*Gbb)/(1-m^2);

  for t=2:nrp
    %*%*%*%*%*%*%*%* obtain environments during development eps_tT and at selection eps_t
    envt = (t>Ut)*delta;
    envtT1 = ((t-tau-1)>Ut)*delta;
    envtT= ((t-tau)>Ut)*delta;
    esigz2t=(Gaa+Gbb*envtT^2)*(2+m)/(1-m^2)/(2-m)+(sigma_e2+vxi*Gbb)/(1-m^2);
    %*%*%*%*%*%*%*%* calculate phenotypic variance at time t
    %*%*%*%*%*%*%*%* gammaStar is selection on z
    egammaStar = 1/(omega2 + esigz2t);

    %*%*%*%*%*%*%*%* calculate change (del) in elevation & plasticity
    ezbar(t) = eabar(t) + ebbar(t)*envtT + m*ezsbr(t-1);
    etheta = A + B*envt;
    etmp=ezbar(t)-etheta;
    emat=[etmp*(1+m); (envtT+m*envtT1)*etmp+(ebbar(t)*(1+m^2)-rho*B)*vxi];
    ebeta = -egammaStar*emat;
    edel = GG * ebeta;

    %*%*%*%*%*%*%*%* calculate optimum phenotype (theta) 
    %*%*%*%*%*%*%*%* ...& mean fitness (Cbar) after first incidence of selection
    efitness(t) = Wmax*sqrt(egammaStar*omega2)*exp(-(egammaStar/2)*((ezbar(t)-etheta)^2+vxi*(ebbar(t)^2*(1+m^2)+B^2-2*ebbar(t)*B*rho)));

    %*%*%*%*%*%*%*%* update elevation & plasticity
    eabar(t+1) = eabar(t) + edel(1);
    ebbar(t+1) = ebbar(t) + edel(2);
    
    %*%*%*%*%*%*%*%* update zbar, then update zstarbar (zsbr) ...
    %*%*%*%*%*%*%*%*    ... to link generation n to generation n-1 
    ezsbr(t) = eabar(t)+envtT*ebbar(t)+m*ezsbr(t-1)-egammaStar*Gbb*(envtT+m*envtT1)*vxi*(ebbar(t)-rho*B)-egammaStar*Gbb*envtT*(ebbar(t)*(1+m^2)-B*rho)*vxi-egammaStar*(Gaa*(1+m)+Gbb*(envtT*(envtT+m*envtT1)+vxi))*etmp;
  end

  %*%*%*%*%*%*%*%* generate plots
  subplot(2,2,1);semilogx(ezbar,'linewidth',2.0,'color',colormap(mm,:));hold on;
  xlabel('generations','fontsize',9,'fontname','TimesNewRoman');ylabel('phenotype','fontname','TimesNewRoman','fontsize',9);axis([1 10^5 -5 30]);
  set(gca,'box','off','xtick',[1 11 111 10^4],'xticklabel',[-10 0 100 10000],'xminortick','off','ytick',[0 20],'yticklabel',{'A' 'A+B  '},'fontsize',9,'fontname','TimesNewRoman');
  subplot(2,2,2);loglog(efitness,'linewidth',2.0,'color',colormap(mm,:));hold on;
  xlabel('generations','fontsize',9,'fontname','TimesNewRoman');ylabel('log(mean fitness)','fontsize',9,'fontname','TimesNewRoman');axis([1 10^5 0 1]);
  set(gca,'box','off','xtick',[1 11 111 10^4],'xticklabel',[-10 0 100 10000],'xminortick','off','ytick',[1],'yticklabel',{'1'},'fontsize',9,'fontname','TimesNewRoman');
  subplot(2,2,3);semilogx(ebbar,'linewidth',2.0,'color',colormap(mm,:));hold on;
  xlabel('generations','fontsize',9,'fontname','TimesNewRoman');ylabel('plasticity','fontsize',9,'fontname','TimesNewRoman');axis([1 10^5 0 2]);
  set(gca,'box','off','xtick',[1 11 111 10^4],'xticklabel',[-10 0 100 10000],'xminortick','off','ytick',[0 0.5 2],'yticklabel',{'0' '  B' 'B'},'fontsize',9,'fontname','TimesNewRoman');
  subplot(2,2,4);semilogx(eabar,'linewidth',2.0,'color',colormap(mm,:));hold on;
  xlabel('generations','fontsize',9,'fontname','TimesNewRoman');ylabel('additive genetic component','fontsize',9,'fontname','TimesNewRoman');axis([1 10^5 -5 20]);
  set(gca,'box','off','xtick',[1 11 111 10^4],'xticklabel',[-10 0 100 10000],'xminortick','off','ytick',[0 20],'yticklabel',{'A' 'A+B  '},'fontsize',9,'fontname','TimesNewRoman');
end