%*%* The code modifies the reaction norm approach of Lande (2009) to include maternal ...
%*%* ... effects (Hoyle & Ezard 2012) and then explores different models of ...
%*%* ... environmental change that are correlated across generations.

%*%* This is the script for the stochastic flipping environment generated by a Poisson process

%*%* This model is explained fully in Ezard et al. (2014). The fitness costs of 
%*%* adaptation via phenotypic plasticity and maternal effects. Func. Ecol.  doi: 10.1111/1365-2435.12207

%*%* This core was published as Hoyle, R.B. & Ezard, T.H.G. (2012). 
%*%* The benefits of maternal effects in novel and in stable environments. 
%*%* J. Roy. Soc. Interface 9, 2403-2413. doi:10.1098/rsif.2012.0183

%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
%*%* start
clear
close

%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
%*%* parameter definitions
%tau=0.25;		% SEE BELOW
rho=0.25;		% environmental auto-correlation
xi_bar=0;		% mean of stationary autocorrelated noise
%sig_xi=2;		% SEE BELOW
omega2=50;		% width of fitness function, 3=> strong stabilizing selection
sigma_e2=.5;	% variance in residual component of phenotypic variation
A=0;			% intercept (elevation) of evolution in reference environment
B=2;			% slope of reaction norm in reference environment
Wmax=1;			% maximal fitness
%m=0.5;			% SEE BELOW
Gaa=0.5;		% variance of elevation, hold %GaaGbbrat = (Gbb*delta^2)/(Gaa+Gbb*delta^2) near 1 in Lande
Gab=0;			% assume zero covariance 
Gba=0; 			% assume zero covariance 
Gbb=0.045;		% variance of plasticity
%delta=10;		% SEE BELOW
nev=1.1e5;		% maximum number of events (draw this many random numbers, then dump anything that happens after nrp)
nrp=1.1e5;		% number of generations

%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
%*%*%* define the things that change
museq = [0.02 0.2]';	% rate of flipping of Poisson process
le =length(museq);
delseq = 7:3:13			% difference between environmental extremes 
ld = length(delseq);
xiseq = 2;				% standard deviation of stationary autocorrelated noise
lx = length(xiseq);
which_env='ps';

%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
%*%*%* loop through xi, delta and 
for p=1:lx
  sig_xi=xiseq(p)
  for q=1:ld
    delta=delseq(q);
    ['Delta is now ' num2str(delta)]
    for n=1:le;
      mu = museq(n);
      muinv=1/mu
      %*%* get environmental sequence
      flipTimes = cumsum(exprnd(1/mu, [nev, 1]));
      tt = sort(repmat([0 flipTimes'], 2, 1));
      tt = tt(tt<nrp);	%truncate to stop at 1e5 generations
      ltt=length(tt);
      flips = repmat([true false false true], 1, ltt/2);
      flips = flips(1:ltt);
      %plot(tt, flips);axis([1000 1100 -.1, 1.1])		%a quick test plot, if needs be

      %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
      %*%*%* Initiate sequence of tau and m values, holding the environmental sequence constant
      tv=.05:.15:.35;		% tau: fraction of a generation between critical period of development and selection 
      lt=length(tv);
      mm=-0.7:.05:0.7;		% m!=0  => maternal effects
      lm=length(mm);
  
      fot=zeros(lm, lt);
      got=zeros(lm, lt);
      aot=zeros(lm, lt);
      bot=zeros(lm, lt);

      pnv=zeros(lm, lt);
      apt=zeros(lm, lt);
      plc=zeros(lm, lt);

      for j=1:lt;
        tau=tv(j)
        for i=1:lm;
          m=mm(i);

          %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
          %*%*%* Run the core
          eabar=zeros(nrp,1);
          ebbar=zeros(nrp,1);
          ezbar=zeros(nrp,1);
          ezsbr=zeros(nrp,1);
          efitness=zeros(nrp,1);

          %*%* initiate
          eabar(1)=(1-m)*A;
          ebbar(1)=rho*B/(1+m^2);
          bsbr(1)=rho*B;
          ezsbr(1)=A;
          ezbar(1)=A;

          vxi=sig_xi^2;
          egaz=Gaa/(1-m/2);
          egbz=(delta/2)*Gbb/(1-m/2);
          esigz2 = Gaa*(2+m)/(1-m^2)/(2-m)+(sigma_e2+vxi*Gbb)/(1-m^2);%'this has been updated
          GG=[Gaa Gab; Gba Gbb];

          %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
          %*%*loop - the guts of the code
          for t=2:nrp
            %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
            %*%* get environments during development eps_tT (env_devel), at selection ...
            %*%* ... eps_t (env_selct) and during development in previous generation (env_ed)
            
            env_ed = (flips(find(tt<(t-1-tau), 1, 'last')) - 0.5)*delta;
            env_devel = (flips(find(tt<(t-tau), 1, 'last')) - 0.5)*delta;
            env_selct = (flips(find(tt<t, 1, 'last')) - 0.5)*delta;

            %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
            %*%*%*%*%*%*%*%* calculate phenotypic variance at time t
            p1 = Gaa + Gbb*env_devel*env_ed;
            esigz2t = (Gaa +Gbb*(env_devel^2) + sigma_e2 + Gbb*vxi + ((2*m)/(2-m))*p1)/(1 - m^2);
            egammaStar = 1/(omega2 + esigz2t);

            %*%*%*%*%*%*%*%* calculate change (del) in elevation & plasticity 
            ezbar(t) = eabar(t) + ebbar(t)*env_devel + m*ezsbr(t-1);
            etheta = A + B*env_selct;
  
            eDisc=ezbar(t)-etheta;
            emat=[(1+m)*eDisc; (env_devel + m*env_ed)*eDisc + ((ebbar(t)*(1+m^2)-rho*B)*vxi)];

            ebeta = -egammaStar*emat;
            GG=[Gaa Gab; Gba Gbb];
            edel = GG * ebeta;

            %*%*%*%*%*%*%*%* calculate fitness components (phenotypic variance, adaptation and plasticity constraint) 
            ephenvar(t)    = sqrt(egammaStar*omega2);
            adapt(t)       = exp(-egammaStar/2 * eDisc^2);
            plasCnstrnt(t) = exp((-(egammaStar*vxi)/2) * (ebbar(t)^2*(1+m^2) + B^2 - 2*ebbar(t)*B*rho));
            efitness(t)    = ephenvar(t)*adapt(t)*plasCnstrnt(t);

            %*%*%*%*%*%*%*%* update elevation & plasticity
            eabar(t+1) = eabar(t) + edel(1);
            ebbar(t+1) = ebbar(t) + edel(2);
    
            %*%*%*%*%*%*%*%* update zbar, then update zstarbar (zsbr) ...
            %*%*%*%*%*%*%*%*    ... to link generation n to generation n-1 
            ezsbr(t) = eabar(t) + env_devel*ebbar(t) + m*ezsbr(t-1)   -   egammaStar*Gbb*(env_devel + m*env_ed)*vxi*(ebbar(t)-rho*B) -   egammaStar*Gbb*env_devel*(ebbar(t)*(1+m^2)-B*rho)*vxi     -   egammaStar*(Gaa*(1+m) + Gbb*(env_devel*(env_devel+m*env_ed) + vxi))*eDisc;
          end
          
          %*%*%* End the core
          %*%*%* NOTE TO SELF: THGE ran this as a separate file 'thge_expec_pois_121119.m'
          %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
                   
          %*%*%*%*%* output some stuff
          fot(i, j) = mean(efitness(1e4:nrp));
          got(i, j) = geomean(efitness(1e4:nrp));
          aot(i, j) = mean(eabar(1e4:nrp));
          bot(i, j) = mean(ebbar(1e4:nrp));
          pnv(i, j) = mean(ephenvar(1e4:nrp));
          apt(i, j) = mean(adapt(1e4:nrp));
          plc(i, j) = mean(plasCnstrnt(1e4:nrp));
         end;
      end;

      %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
      %*%*%* output everything to a matlab and text files
      savefile =[date '_pois_del' num2str(delta) '_mu'  num2str(1/museq(n)) '_xi' num2str(sig_xi) '.mat'];
      save(savefile, 'fot', 'got', 'pnv', 'apt', 'plc', 'aot', 'bot')

      fid = fopen([date '_pois_del' num2str(delta) '_mu'  num2str(1/museq(n)) '_xi' num2str(sig_xi) '_fot.txt'], 'w');
      fprintf(fid, '%12.8f\n', fot);
      fclose(fid); 
      fid = fopen([date '_pois_del' num2str(delta) '_mu'  num2str(1/museq(n)) '_xi' num2str(sig_xi) '_got.txt'], 'w');
      fprintf(fid, '%12.8f\n', got);
      fclose(fid);       
      fid = fopen([date '_pois_del' num2str(delta) '_mu'  num2str(1/museq(n)) '_xi' num2str(sig_xi) '_pnv.txt'], 'w');
      fprintf(fid, '%12.8f\n', pnv);
      fclose(fid);
      fid = fopen([date '_pois_del' num2str(delta) '_mu'  num2str(1/museq(n)) '_xi' num2str(sig_xi) '_apt.txt'], 'w');
      fprintf(fid, '%12.8f\n', apt);
      fclose(fid);      
      fid = fopen([date '_pois_del' num2str(delta) '_mu'  num2str(1/museq(n)) '_xi' num2str(sig_xi) '_plc.txt'], 'w');
      fprintf(fid, '%12.8f\n', plc);
      fclose(fid);      
    end;
  end;
end;
